/* libspiasm.c -  a fast spi interface library project
   Test harness for the spiasm library. 
   Interface to DS1620 digital thermometer
   Based on SPI_Asm_Demo.spin
  
 Use pasm modified from SPI_Asm.spin propeller tool library object.
  T Montemarano (with a lot of help)
  
   NOTES:
 Uses spin file xSPIAsm.spin,
   This version does run correctly.  
  
  The SPI device called in main() is a DS1620 Digital Thermometer.  
  
  */

#include "simpletools.h"                      // Library includes
#include "spiasm.h"
  
/* ******************************************************
   libspipasm DS1620.c  uses DS1620 as SPI device


    DS1620 SPI data in/out <-> Prop P0 with 330 ohm resistor, LSB first LPRE
    Clock <- Prop P1  fall then rise  State = 1
    Reset <- Prop P2  high starts transfer, low terminates
    After configuration data sent to device pause 10ms.
    Temperature conversion takes 1 sec
*/

#define cstate 1                  // 0 = start clock low, 1 = start clock high
#define clockd_ns 1000      // Actual delay in nanoseconds (1000ns = 1 usec), not less than 300ns
                                         // ClockDelay, actual clock delay = 300ns + (cdelay-1) * 50ns
                                         // e.g. cdelay of 15 gives actual clock delay = 1 usec
                                         // cdelay = ((clockd_ns) - 250ns)) / 50ns
                                         // e.g. for 2 usec (i.e. 2000 ns): cdelay = 35
//  ** Propeller pins 
#define IOPIN 0
#define CLKPIN 1
#define RESET 2

spia_t *spia; 
  
int main()                                    // Main function
{ 
// ** DS1620 commands
  int wrcfg = 0x0C;     // Write config register
  int startc = 0xEE;    // Start Conversion
  int rdtemp = 0xAA;    // Read Temperature

  int degC;
  int degF;

  spia = spi_start(clockd_ns, cstate);    // Launch, get handles, set ClockDelay & ClockState

  high(RESET);                  // enable DS1620 data transfer

// void spi_out(spia_t *device, int DQ, int CLK, int MD, int BIT, int VLU);

  spi_out(spia, IOPIN, CLKPIN, LFIRST, 8, wrcfg);   // request Confguration write
  spi_out(spia, IOPIN, CLKPIN, LFIRST, 8, 0b10);    // configure for CPU / Free Run mode
  low(RESET);                   // release DS1620

  pause(10);                    // allow time for DS1620 to save configuration

  high(RESET);
  spi_out(spia, IOPIN, CLKPIN, LFIRST, 8, startc);  // request Start Conversion
  pause(10);                        // need to wait before setting RESET low
  low(RESET);

  while(1)  
  {
    pause(1000);            // allow time for data conversion
    high(RESET);
    spi_out(spia, IOPIN, CLKPIN, LFIRST, 8, rdtemp);    // request to Read the Temperature
    degC = spi_in(spia, IOPIN, CLKPIN, LPRE, 9);         // read 9 bit temperature
    low(RESET);

    //printi("%b     ", degC);
    if(degC & 0b100000000) degC = degC | 0xFFFFFF00; // degC <0, convert to 32bit signed int

    degC *= 5;                            // data is in 1/2 degree increments, make degC = 10 x
    degF = degC * 9 / 5 + 320;
    printi(" %d.%d deg C   ", degC/10, degC - (degC/10 * 10));
    printi(" %d.%d deg F\n", degF/10, degF - (degF/10 * 10));
  }
}   // end main
